@@ -36,7 +36,7 @@
 #include <sys/wait.h>
 #endif
 
-RCSID("$Id$");
+extern int daemon_child;
 
 struct kadm_port {
     char *port;
@@ -45,12 +45,12 @@ struct kadm_port {
 } *kadm_ports;
 
 static void
-add_kadm_port(krb5_context context, const char *service, unsigned int port)
+add_kadm_port(krb5_context contextp, const char *service, unsigned int port)
 {
     struct kadm_port *p;
     p = malloc(sizeof(*p));
     if(p == NULL) {
-	krb5_warnx(context, "failed to allocate %lu bytes\n",
+	krb5_warnx(contextp, "failed to allocate %lu bytes\n",
 		   (unsigned long)sizeof(*p));
 	return;
     }
@@ -63,9 +63,9 @@ add_kadm_port(krb5_context context, const char *service, unsigned int port)
 }
 
 static void
-add_standard_ports (krb5_context context)
+add_standard_ports (krb5_context contextp)
 {
-    add_kadm_port(context, "kerberos-adm", 749);
+    add_kadm_port(contextp, "kerberos-adm", 749);
 }
 
 /*
@@ -75,15 +75,15 @@ add_standard_ports (krb5_context context)
  */
 
 void
-parse_ports(krb5_context context, const char *str)
+parse_ports(krb5_context contextp, const char *str)
 {
     char p[128];
 
     while(strsep_copy(&str, " \t", p, sizeof(p)) != -1) {
 	if(strcmp(p, "+") == 0)
-	    add_standard_ports(context);
+	    add_standard_ports(contextp);
 	else
-	    add_kadm_port(context, p, 0);
+	    add_kadm_port(contextp, p, 0);
     }
 }
 
@@ -94,7 +94,12 @@ static RETSIGTYPE
 sigchld(int sig)
 {
     int status;
-    waitpid(-1, &status, 0);
+    /*
+     * waitpid() is async safe. will return -1 or 0 on no more zombie
+     * children
+     */
+    while ((waitpid(-1, &status, WNOHANG)) > 0)
+	;
     SIGRETURN(0);
 }
 
@@ -117,73 +122,77 @@ terminate(int sig)
 }
 
 static int
-spawn_child(krb5_context context, int *socks,
+spawn_child(krb5_context contextp, int *socks,
 	    unsigned int num_socks, int this_sock)
 {
-    int e, i;
+    int e;
+    size_t i;
     struct sockaddr_storage __ss;
     struct sockaddr *sa = (struct sockaddr *)&__ss;
     socklen_t sa_size = sizeof(__ss);
-    int s;
+    krb5_socket_t s;
     pid_t pid;
     krb5_address addr;
     char buf[128];
     size_t buf_len;
 
     s = accept(socks[this_sock], sa, &sa_size);
-    if(s < 0) {
-	krb5_warn(context, errno, "accept");
+    if(rk_IS_BAD_SOCKET(s)) {
+	krb5_warn(contextp, rk_SOCK_ERRNO, "accept");
 	return 1;
     }
-    e = krb5_sockaddr2address(context, sa, &addr);
+    e = krb5_sockaddr2address(contextp, sa, &addr);
     if(e)
-	krb5_warn(context, e, "krb5_sockaddr2address");
+	krb5_warn(contextp, e, "krb5_sockaddr2address");
     else {
 	e = krb5_print_address (&addr, buf, sizeof(buf),
 				&buf_len);
 	if(e)
-	    krb5_warn(context, e, "krb5_print_address");
+	    krb5_warn(contextp, e, "krb5_print_address");
 	else
-	    krb5_warnx(context, "connection from %s", buf);
-	krb5_free_address(context, &addr);
+	    krb5_warnx(contextp, "connection from %s", buf);
+	krb5_free_address(contextp, &addr);
     }
 
     pid = fork();
     if(pid == 0) {
 	for(i = 0; i < num_socks; i++)
-	    close(socks[i]);
+	    rk_closesocket(socks[i]);
 	dup2(s, STDIN_FILENO);
 	dup2(s, STDOUT_FILENO);
 	if(s != STDIN_FILENO && s != STDOUT_FILENO)
-	    close(s);
+	    rk_closesocket(s);
 	return 0;
     } else {
-	close(s);
+	rk_closesocket(s);
     }
     return 1;
 }
 
-static int
-wait_for_connection(krb5_context context,
-		    int *socks, unsigned int num_socks)
+static void
+wait_for_connection(krb5_context contextp,
+		    krb5_socket_t *socks, unsigned int num_socks)
 {
     unsigned int i;
     int e;
     fd_set orig_read_set, read_set;
-    int max_fd = -1;
+    int status, max_fd = -1;
 
     FD_ZERO(&orig_read_set);
 
     for(i = 0; i < num_socks; i++) {
+#ifdef FD_SETSIZE
 	if (socks[i] >= FD_SETSIZE)
 	    errx (1, "fd too large");
+#endif
 	FD_SET(socks[i], &orig_read_set);
 	max_fd = max(max_fd, socks[i]);
     }
 
     pgrp = getpid();
 
-    if(setpgid(0, pgrp) < 0)
+    /* systemd may cause setpgid to fail with EPERM */
+    if(setpgid(0, pgrp) < 0 && errno != EPERM)
 	err(1, "setpgid");
 
     signal(SIGTERM, terminate);
@@ -193,41 +202,43 @@ wait_for_connection(krb5_context context,
     while (term_flag == 0) {
 	read_set = orig_read_set;
 	e = select(max_fd + 1, &read_set, NULL, NULL, NULL);
-	if(e < 0) {
-	    if(errno != EINTR)
-		krb5_warn(context, errno, "select");
+	if(rk_IS_SOCKET_ERROR(e)) {
+	    if(rk_SOCK_ERRNO != EINTR)
+		krb5_warn(contextp, rk_SOCK_ERRNO, "select");
 	} else if(e == 0)
-	    krb5_warnx(context, "select returned 0");
+	    krb5_warnx(contextp, "select returned 0");
 	else {
 	    for(i = 0; i < num_socks; i++) {
 		if(FD_ISSET(socks[i], &read_set))
-		    if(spawn_child(context, socks, num_socks, i) == 0)
-			return 0;
+		    if(spawn_child(contextp, socks, num_socks, i) == 0)
+			return;
 	    }
 	}
     }
     signal(SIGCHLD, SIG_IGN);
-    while(1) {
-	int status;
-	pid_t pid;
-	pid = waitpid(-1, &status, 0);
-	if(pid == -1 && errno == ECHILD)
-	    break;
-    }
+
+    while ((waitpid(-1, &status, WNOHANG)) > 0)
+	;
+
     exit(0);
 }
 
 
-int
-start_server(krb5_context context)
+void
+start_server(krb5_context contextp, const char *port_str)
 {
     int e;
     struct kadm_port *p;
 
-    int *socks = NULL, *tmp;
+    krb5_socket_t *socks = NULL, *tmp;
     unsigned int num_socks = 0;
     int i;
 
+    if (port_str == NULL)
+	port_str = "+";
+
+    parse_ports(contextp, port_str);
+
     for(p = kadm_ports; p; p = p->next) {
 	struct addrinfo hints, *ai, *ap;
 	char portstr[32];
@@ -242,7 +253,7 @@ start_server(krb5_context context)
 	}
 
 	if(e) {
-	    krb5_warn(context, krb5_eai_to_heim_errno(e, errno),
+	    krb5_warn(contextp, krb5_eai_to_heim_errno(e, errno),
 		      "%s", portstr);
 	    continue;
 	}
@@ -251,29 +262,30 @@ start_server(krb5_context context)
 	    i++;
 	tmp = realloc(socks, (num_socks + i) * sizeof(*socks));
 	if(tmp == NULL) {
-	    krb5_warnx(context, "failed to reallocate %lu bytes",
+	    krb5_warnx(contextp, "failed to reallocate %lu bytes",
 		       (unsigned long)(num_socks + i) * sizeof(*socks));
+            freeaddrinfo(ai);
 	    continue;
 	}
 	socks = tmp;
 	for(ap = ai; ap; ap = ap->ai_next) {
-	    int s = socket(ap->ai_family, ap->ai_socktype, ap->ai_protocol);
-	    if(s < 0) {
-		krb5_warn(context, errno, "socket");
+	    krb5_socket_t s = socket(ap->ai_family, ap->ai_socktype, ap->ai_protocol);
+	    if(rk_IS_BAD_SOCKET(s)) {
+		krb5_warn(contextp, rk_SOCK_ERRNO, "socket");
 		continue;
 	    }
 
 	    socket_set_reuseaddr(s, 1);
 	    socket_set_ipv6only(s, 1);
 
-	    if (bind (s, ap->ai_addr, ap->ai_addrlen) < 0) {
-		krb5_warn(context, errno, "bind");
-		close(s);
+	    if (rk_IS_SOCKET_ERROR(bind (s, ap->ai_addr, ap->ai_addrlen))) {
+		krb5_warn(contextp, rk_SOCK_ERRNO, "bind");
+		rk_closesocket(s);
 		continue;
 	    }
-	    if (listen (s, SOMAXCONN) < 0) {
-		krb5_warn(context, errno, "listen");
-		close(s);
+	    if (rk_IS_SOCKET_ERROR(listen (s, SOMAXCONN))) {
+		krb5_warn(contextp, rk_SOCK_ERRNO, "listen");
+		rk_closesocket(s);
 		continue;
 	    }
 	    socks[num_socks++] = s;
@@ -281,6 +293,10 @@ start_server(krb5_context context)
 	freeaddrinfo (ai);
     }
     if(num_socks == 0)
-	krb5_errx(context, 1, "no sockets to listen to - exiting");
-    return wait_for_connection(context, socks, num_socks);
+	krb5_errx(contextp, 1, "no sockets to listen to - exiting");
+
+    roken_detach_finish(NULL, daemon_child);
+
+    wait_for_connection(contextp, socks, num_socks);
+    free(socks);
 }
