@@ -59,7 +59,7 @@ kcm_op_noop(krb5_context context,
 {
     KCM_LOG_REQUEST(context, client, opcode);
 
-    return 0;	
+    return 0;
 }
 
 /*
@@ -108,7 +108,7 @@ kcm_op_get_name(krb5_context context,
 
 /*
  * Request:
- *	
+ *
  * Response:
  *	NameZ
  */
@@ -139,9 +139,9 @@ kcm_op_gen_new(krb5_context context,
  * Request:
  *	NameZ
  *	Principal
- *	
+ *
  * Response:
- *	
+ *
  */
 static krb5_error_code
 kcm_op_initialize(krb5_context context,
@@ -205,9 +205,9 @@ kcm_op_initialize(krb5_context context,
 /*
  * Request:
  *	NameZ
- *	
+ *
  * Response:
- *	
+ *
  */
 static krb5_error_code
 kcm_op_destroy(krb5_context context,
@@ -238,9 +238,9 @@ kcm_op_destroy(krb5_context context,
  * Request:
  *	NameZ
  *	Creds
- *	
+ *
  * Response:
- *	
+ *
  */
 static krb5_error_code
 kcm_op_store(krb5_context context,
@@ -298,7 +298,7 @@ kcm_op_store(krb5_context context,
  *
  * Response:
  *	Creds
- *	
+ *
  */
 static krb5_error_code
 kcm_op_retrieve(krb5_context context,
@@ -432,7 +432,7 @@ kcm_op_get_principal(krb5_context context,
  *
  * Response:
  *	UUIDs
- *	
+ *
  */
 static krb5_error_code
 kcm_op_get_cred_uuid_list(krb5_context context,
@@ -535,7 +535,7 @@ kcm_op_get_cred_by_uuid(krb5_context context,
  *	MatchCreds
  *
  * Response:
- *	
+ *
  */
 static krb5_error_code
 kcm_op_remove_cred(krb5_context context,
@@ -593,7 +593,7 @@ kcm_op_remove_cred(krb5_context context,
  *	Flags
  *
  * Response:
- *	
+ *
  */
 static krb5_error_code
 kcm_op_set_flags(krb5_context context,
@@ -640,7 +640,7 @@ kcm_op_set_flags(krb5_context context,
  *	GID
  *
  * Response:
- *	
+ *
  */
 static krb5_error_code
 kcm_op_chown(krb5_context context,
@@ -694,7 +694,7 @@ kcm_op_chown(krb5_context context,
  *	Mode
  *
  * Response:
- *	
+ *
  */
 static krb5_error_code
 kcm_op_chmod(krb5_context context,
@@ -824,7 +824,7 @@ kcm_op_get_initial_ticket(krb5_context context,
 
     if (ret != 0) {
 	krb5_free_principal(context, server);
-	krb5_free_keyblock(context, &key);
+	krb5_free_keyblock_contents(context, &key);
     }
 
     kcm_release_ccache(context, ccache);
@@ -948,6 +948,13 @@ kcm_op_move_cache(krb5_context context,
 	return ret;
     }
 
+    /* move to ourself is simple, done! */
+    if (strcmp(oldname, newname) == 0) {
+	free(oldname);
+	free(newname);
+	return 0;
+    }
+
     ret = kcm_ccache_resolve_client(context, client, opcode, oldname, &oldid);
     if (ret) {
 	free(oldname);
@@ -1063,6 +1070,7 @@ kcm_op_get_default_cache(krb5_context context,
     krb5_error_code ret;
     const char *name = NULL;
     char *n = NULL;
+    int aret;
 
     KCM_LOG_REQUEST(context, client, opcode);
 
@@ -1076,8 +1084,9 @@ kcm_op_get_default_cache(krb5_context context,
 	name = n = kcm_ccache_first_name(client);
 
     if (name == NULL) {
-	asprintf(&n, "%d", (int)client->uid);
-	name = n;
+	aret = asprintf(&n, "%d", (int)client->uid);
+	if (aret != -1)
+	    name = n;
     }
     if (name == NULL)
 	return ENOMEM;
@@ -1128,17 +1137,19 @@ kcm_op_set_default_cache(krb5_context context,
     }
     if (c == NULL) {
 	c = malloc(sizeof(*c));
-	if (c == NULL)
+	if (c == NULL) {
+            free(name);
 	    return ENOMEM;
+        }
 	c->session = client->session;
 	c->uid = client->uid;
-	c->name = strdup(name);
+	c->name = name;
 
 	c->next = default_caches;
 	default_caches = c;
     } else {
 	free(c->name);
-	c->name = strdup(name);
+	c->name = name;
     }
 
     return 0;
@@ -1250,7 +1261,7 @@ find_ntlm_cred(const char *user, const char *domain, kcm_client *client)
     struct kcm_ntlm_cred *c;
 
     for (c = ntlm_head; c != NULL; c = c->next)
-	if ((user[0] == '\0' || strcmp(user, c->user) == 0) && 
+	if ((user[0] == '\0' || strcmp(user, c->user) == 0) &&
 	    (domain == NULL || strcmp(domain, c->domain) == 0) &&
 	    kcm_is_same_session(client, c->uid, c->session))
 	    return c;
@@ -1441,7 +1452,7 @@ kcm_op_do_ntlm(krb5_context context,
     memset(&type3, 0, sizeof(type3));
     sessionkey.data = NULL;
     sessionkey.length = 0;
-    
+
     ret = krb5_ret_stringz(request, &user);
     if (ret)
 	goto error;
@@ -1482,40 +1493,40 @@ kcm_op_do_ntlm(krb5_context context,
     type3.flags = type2.flags;
     type3.targetname = type2.targetname;
     type3.ws = rk_UNCONST("workstation");
-    
+
     /*
      * NTLM Version 1 if no targetinfo buffer.
      */
-    
+
     if (1 || type2.targetinfo.length == 0) {
-	struct ntlm_buf sessionkey;
-	
+	struct ntlm_buf tmpsesskey;
+
 	if (type2.flags & NTLM_NEG_NTLM2_SESSION) {
 	    unsigned char nonce[8];
-	    
+
 	    if (RAND_bytes(nonce, sizeof(nonce)) != 1) {
 		ret = EINVAL;
 		goto error;
 	    }
-	    
+
 	    ret = heim_ntlm_calculate_ntlm2_sess(nonce,
-						 type2.challange,
+						 type2.challenge,
 						 c->nthash.data,
 						 &type3.lm,
 						 &type3.ntlm);
 	} else {
 	    ret = heim_ntlm_calculate_ntlm1(c->nthash.data,
 					    c->nthash.length,
-					    type2.challange,
+					    type2.challenge,
 					    &type3.ntlm);
-	    
+
 	}
 	if (ret)
 	    goto error;
-	
+
 	ret = heim_ntlm_build_ntlm1_master(c->nthash.data,
 					   c->nthash.length,
-					   &sessionkey,
+					   &tmpsesskey,
 					   &type3.sessionkey);
 	if (ret) {
 	    if (type3.lm.data)
@@ -1525,7 +1536,7 @@ kcm_op_do_ntlm(krb5_context context,
 	    goto error;
 	}
 
-	free(sessionkey.data);
+	free(tmpsesskey.data);
 	if (ret) {
 	    if (type3.lm.data)
 		free(type3.lm.data);
@@ -1534,14 +1545,14 @@ kcm_op_do_ntlm(krb5_context context,
 	    goto error;
 	}
 	flags |= NTLM_FLAG_SESSIONKEY;
-#if 0	
+#if 0
     } else {
 	struct ntlm_buf sessionkey;
 	unsigned char ntlmv2[16];
 	struct ntlm_targetinfo ti;
-	
+
 	/* verify infotarget */
-	
+
 	ret = heim_ntlm_decode_targetinfo(&type2.targetinfo, 1, &ti);
 	if(ret) {
 	    _gss_ntlm_delete_sec_context(minor_status,
@@ -1549,19 +1560,19 @@ kcm_op_do_ntlm(krb5_context context,
 	    *minor_status = ret;
 	    return GSS_S_FAILURE;
 	}
-	
+
 	if (ti.domainname && strcmp(ti.domainname, name->domain) != 0) {
 	    _gss_ntlm_delete_sec_context(minor_status,
 					 context_handle, NULL);
 	    *minor_status = EINVAL;
 	    return GSS_S_FAILURE;
 	}
-	
+
 	ret = heim_ntlm_calculate_ntlm2(ctx->client->key.data,
 					ctx->client->key.length,
 					type3.username,
 					name->domain,
-					type2.challange,
+					type2.challenge,
 					&type2.targetinfo,
 					ntlmv2,
 					&type3.ntlm);
@@ -1571,7 +1582,7 @@ kcm_op_do_ntlm(krb5_context context,
 	    *minor_status = ret;
 	    return GSS_S_FAILURE;
 	}
-	
+
 	ret = heim_ntlm_build_ntlm1_master(ntlmv2, sizeof(ntlmv2),
 					   &sessionkey,
 					   &type3.sessionkey);
@@ -1582,10 +1593,10 @@ kcm_op_do_ntlm(krb5_context context,
 	    *minor_status = ret;
 	    return GSS_S_FAILURE;
 	}
-	
+
 	flags |= NTLM_FLAG_NTLM2_SESSION |
 	         NTLM_FLAG_SESSION;
-	
+
 	if (type3.flags & NTLM_NEG_KEYEX)
 	    flags |= NTLM_FLAG_KEYEX;
 
@@ -1600,7 +1611,7 @@ kcm_op_do_ntlm(krb5_context context,
 	}
 #endif
     }
-    
+
 #if 0
     if (flags & NTLM_FLAG_NTLM2_SESSION) {
 	_gss_ntlm_set_key(&ctx->u.v2.send, 0, (ctx->flags & NTLM_NEG_KEYEX),
@@ -1620,10 +1631,10 @@ kcm_op_do_ntlm(krb5_context context,
     }
 #endif
 
-    ret = heim_ntlm_encode_type3(&type3, &ndata);
+    ret = heim_ntlm_encode_type3(&type3, &ndata, NULL);
     if (ret)
 	goto error;
-	
+
     data.data = ndata.data;
     data.length = ndata.length;
     ret = krb5_store_data(response, data);
