@@ -34,25 +34,33 @@
  */
 
 #include "kuser_locl.h"
-#include "rtbl.h"
 #include "parse_units.h"
+#include "heimtools-commands.h"
+
+static char*
+printable_time_internal(time_t t, int x)
+{
+    static char s[128];
+    char *p;
+
+    if ((p = ctime(&t)) == NULL)
+	strlcpy(s, "?", sizeof(s));
+    else
+	strlcpy(s, p + 4, sizeof(s));
+    s[x] = 0;
+    return s;
+}
 
 static char*
 printable_time(time_t t)
 {
-    static char s[128];
-    strlcpy(s, ctime(&t)+ 4, sizeof(s));
-    s[15] = 0;
-    return s;
+    return printable_time_internal(t, 20);
 }
 
 static char*
 printable_time_long(time_t t)
 {
-    static char s[128];
-    strlcpy(s, ctime(&t)+ 4, sizeof(s));
-    s[20] = 0;
-    return s;
+    return printable_time_internal(t, 20);
 }
 
 #define COL_ISSUED		NP_("  Issued","")
@@ -121,13 +129,18 @@ print_cred(krb5_context context, krb5_creds *cred, rtbl_t ct, int do_flags)
 }
 
 static void
-print_cred_verbose(krb5_context context, krb5_creds *cred)
+print_cred_verbose(krb5_context context, krb5_creds *cred, int do_json)
 {
-    int j;
+    size_t j;
     char *str;
     krb5_error_code ret;
     krb5_timestamp sec;
 
+    if (do_json) { /* XXX support more json formating later */
+	printf("{ \"verbose-supported\" : false }");
+	return;
+    }
+
     krb5_timeofday (context, &sec);
 
     ret = krb5_unparse_name(context, cred->server, &str);
@@ -135,18 +148,18 @@ print_cred_verbose(krb5_context context, krb5_creds *cred)
 	exit(1);
     printf(N_("Server: %s\n", ""), str);
     free (str);
-    
+
     ret = krb5_unparse_name(context, cred->client, &str);
     if(ret)
 	exit(1);
     printf(N_("Client: %s\n", ""), str);
     free (str);
     
-    {
+    if (!krb5_is_config_principal(context, cred->client)) {
 	Ticket t;
 	size_t len;
 	char *s;
-	
+
 	decode_Ticket(cred->ticket.data, cred->ticket.length, &t, &len);
 	ret = krb5_enctype_to_string(context, t.enc_part.etype, &s);
 	printf(N_("Ticket etype: ", ""));
@@ -187,7 +200,7 @@ print_cred_verbose(krb5_context context, krb5_creds *cred)
 	       printable_time_long(cred->times.renew_till));
     {
 	char flags[1024];
-	unparse_flags(TicketFlags2int(cred->flags.b), 
+	unparse_flags(TicketFlags2int(cred->flags.b),
 		      asn1_TicketFlags_units(),
 		      flags, sizeof(flags));
 	printf(N_("Ticket flags: %s\n", ""), flags);
@@ -200,7 +213,7 @@ print_cred_verbose(krb5_context context, krb5_creds *cred)
 	    if(j) printf(", ");
 	    ret = krb5_print_address(&cred->addresses.val[j],
 				     buf, sizeof(buf), &len);
-	
+
 	    if(ret == 0)
 		printf("%s", buf);
 	}
@@ -211,7 +224,7 @@ print_cred_verbose(krb5_context context, krb5_creds *cred)
 }
 
 /*
- * Print all tickets in `ccache' on stdout, verbosily iff do_verbose.
+ * Print all tickets in `ccache' on stdout, verbosely if do_verbose.
  */
 
 static void
@@ -220,10 +233,11 @@ print_tickets (krb5_context context,
 	       krb5_principal principal,
 	       int do_verbose,
 	       int do_flags,
-	       int do_hidden)
+	       int do_hidden,
+	       int do_json)
 {
+    char *str, *name, *fullname;
     krb5_error_code ret;
-    char *str, *name;
     krb5_cc_cursor cursor;
     krb5_creds creds;
     krb5_deltat sec;
@@ -234,45 +248,52 @@ print_tickets (krb5_context context,
     if (ret)
 	krb5_err (context, 1, ret, "krb5_unparse_name");
 
-    printf ("%17s: %s:%s\n",
-	    N_("Credentials cache", ""),
-	    krb5_cc_get_type(context, ccache),
-	    krb5_cc_get_name(context, ccache));
-    printf ("%17s: %s\n", N_("Principal", ""), str);
+    ret = krb5_cc_get_full_name(context, ccache, &fullname);
+    if (ret)
+	krb5_err (context, 1, ret, "krb5_cc_get_full_name");
 
-    ret = krb5_cc_get_friendly_name(context, ccache, &name);
-    if (ret == 0) {
-	if (strcmp(name, str) != 0)
-	    printf ("%17s: %s\n", N_("Friendly name", ""), name);
-	free(name);
-    }
-    free (str);
-
-    if(do_verbose)
-	printf ("%17s: %d\n", N_("Cache version", ""),
-		krb5_cc_get_version(context, ccache));
-
-    ret = krb5_cc_get_kdc_offset(context, ccache, &sec);
-
-    if (ret == 0 && do_verbose && sec != 0) {
-	char buf[BUFSIZ];
-	int val;
-	int sig;
-
-	val = sec;
-	sig = 1;
-	if (val < 0) {
-	    sig = -1;
-	    val = -val;
+    if (!do_json) {
+	printf ("%17s: %s\n", N_("Credentials cache", ""), fullname);
+	printf ("%17s: %s\n", N_("Principal", ""), str);
+	
+	ret = krb5_cc_get_friendly_name(context, ccache, &name);
+	if (ret == 0) {
+	    if (strcmp(name, str) != 0)
+		printf ("%17s: %s\n", N_("Friendly name", ""), name);
+	    free(name);
 	}
 	
-	unparse_time (val, buf, sizeof(buf));
+	if(do_verbose) {
+	    printf ("%17s: %d\n", N_("Cache version", ""),
+		    krb5_cc_get_version(context, ccache));
+	} else {
+	    krb5_cc_set_flags(context, ccache, KRB5_TC_NOTICKET);
+	}
+	
+	ret = krb5_cc_get_kdc_offset(context, ccache, &sec);
+	
+	if (ret == 0 && do_verbose && sec != 0) {
+	    char buf[BUFSIZ];
+	    int val;
+	    int sig;
+	    
+	    val = (int)sec;
+	    sig = 1;
+	    if (val < 0) {
+		sig = -1;
+		val = -val;
+	    }
+	    
+	    unparse_time (val, buf, sizeof(buf));
 
-	printf ("%17s: %s%s\n", N_("KDC time offset", ""),
-		sig == -1 ? "-" : "", buf);
+	    printf ("%17s: %s%s\n", N_("KDC time offset", ""),
+		    sig == -1 ? "-" : "", buf);
+	}
+	printf("\n");
+    } else {
+	printf ("{ \"cache\" : \"%s\", \"principal\" : \"%s\", ", fullname, str);
     }
-
-    printf("\n");
+    free(str);
 
     ret = krb5_cc_start_seq_get (context, ccache, &cursor);
     if (ret)
@@ -286,7 +307,13 @@ print_tickets (krb5_context context,
 	    rtbl_add_column(ct, COL_FLAGS, 0);
 	rtbl_add_column(ct, COL_PRINCIPAL, 0);
 	rtbl_set_separator(ct, "  ");
+	if (do_json) {
+	    rtbl_set_flags(ct, RTBL_JSON);
+	    printf("\"tickets\" : ");
+	}
     }
+    if (do_verbose && do_json)
+	printf("\"tickets\" : [");
     while ((ret = krb5_cc_next_cred (context,
 				     ccache,
 				     &cursor,
@@ -294,7 +321,7 @@ print_tickets (krb5_context context,
 	if (!do_hidden && krb5_is_config_principal(context, creds.server)) {
 	    ;
 	}else if(do_verbose){
-	    print_cred_verbose(context, &creds);
+	    print_cred_verbose(context, &creds, do_json);
 	}else{
 	    print_cred(context, &creds, ct, do_flags);
 	}
@@ -309,6 +336,11 @@ print_tickets (krb5_context context,
 	rtbl_format(ct, stdout);
 	rtbl_destroy(ct);
     }
+    if (do_json) {
+	if (do_verbose)
+	    printf("]");
+	printf("}");
+    }
 }
 
 /*
@@ -317,43 +349,21 @@ print_tickets (krb5_context context,
  */
 
 static int
-check_for_tgt (krb5_context context,
-	       krb5_ccache ccache,
-	       krb5_principal principal,
-	       time_t *expiration)
+check_expiration(krb5_context context,
+		 krb5_ccache ccache,
+		 time_t *expiration)
 {
     krb5_error_code ret;
-    krb5_creds pattern;
-    krb5_creds creds;
-    krb5_const_realm client_realm;
-    int expired;
+    time_t t;
 
-    krb5_cc_clear_mcred(&pattern);
-
-    client_realm = krb5_principal_get_realm(context, principal);
-
-    ret = krb5_make_principal (context, &pattern.server,
-			       client_realm, KRB5_TGS_NAME, client_realm, NULL);
-    if (ret)
-	krb5_err (context, 1, ret, "krb5_make_principal");
-    pattern.client = principal;
-
-    ret = krb5_cc_retrieve_cred (context, ccache, 0, &pattern, &creds);
-    krb5_free_principal (context, pattern.server);
-    if (ret) {
-	if (ret == KRB5_CC_END)
-	    return 1;
-	krb5_err (context, 1, ret, "krb5_cc_retrieve_cred");
-    }
-
-    expired = time(NULL) > creds.times.endtime;
+    ret = krb5_cc_get_lifetime(context, ccache, &t);
+    if (ret || t == 0)
+	return 1;
 
     if (expiration)
-	*expiration = creds.times.endtime;
+	*expiration = time(NULL) + t;
 
-    krb5_free_cred_contents (context, &creds);
-
-    return expired;
+    return 0;
 }
 
 /*
@@ -436,7 +446,8 @@ display_tokens(int do_verbose)
 static int
 display_v5_ccache (krb5_context context, krb5_ccache ccache,
 		   int do_test, int do_verbose,
-		   int do_flags, int do_hidden)
+		   int do_flags, int do_hidden,
+		   int do_json)
 {
     krb5_error_code ret;
     krb5_principal principal;
@@ -445,6 +456,10 @@ display_v5_ccache (krb5_context context, krb5_ccache ccache,
 
     ret = krb5_cc_get_principal (context, ccache, &principal);
     if (ret) {
+	if (do_json) {
+	    printf("{}");
+	    return 0;
+	}
 	if(ret == ENOENT) {
 	    if (!do_test)
 		krb5_warnx(context, N_("No ticket file: %s", ""),
@@ -454,10 +469,10 @@ display_v5_ccache (krb5_context context, krb5_ccache ccache,
 	    krb5_err (context, 1, ret, "krb5_cc_get_principal");
     }
     if (do_test)
-	exit_status = check_for_tgt (context, ccache, principal, NULL);
+	exit_status = check_expiration(context, ccache, NULL);
     else
 	print_tickets (context, ccache, principal, do_verbose,
-		       do_flags, do_hidden);
+		       do_flags, do_hidden, do_json);
 
     ret = krb5_cc_close (context, ccache);
     if (ret)
@@ -473,9 +488,9 @@ display_v5_ccache (krb5_context context, krb5_ccache ccache,
  */
 
 static int
-list_caches(krb5_context context)
+list_caches(krb5_context context, struct klist_options *opt)
 {
-    krb5_cc_cache_cursor cursor;
+    krb5_cccol_cursor cursor;
     const char *cdef_name;
     char *def_name;
     krb5_error_code ret;
@@ -487,51 +502,55 @@ list_caches(krb5_context context)
 	krb5_errx(context, 1, "krb5_cc_default_name");
     def_name = strdup(cdef_name);
 
-    ret = krb5_cc_cache_get_first (context, NULL, &cursor);
-    if (ret == KRB5_CC_NOSUPP)
+    ret = krb5_cccol_cursor_new(context, &cursor);
+    if (ret == KRB5_CC_NOSUPP) {
+        free(def_name);
 	return 0;
+    }
     else if (ret)
 	krb5_err (context, 1, ret, "krb5_cc_cache_get_first");
 
     ct = rtbl_create();
+    rtbl_add_column(ct, COL_DEFCACHE, 0);
     rtbl_add_column(ct, COL_NAME, 0);
     rtbl_add_column(ct, COL_CACHENAME, 0);
     rtbl_add_column(ct, COL_EXPIRES, 0);
     rtbl_add_column(ct, COL_DEFCACHE, 0);
     rtbl_set_prefix(ct, "   ");
-    rtbl_set_column_prefix(ct, COL_NAME, "");
+    rtbl_set_column_prefix(ct, COL_DEFCACHE, "");
+    rtbl_set_column_prefix(ct, COL_NAME, " ");
+    if (opt->json_flag)
+	rtbl_set_flags(ct, RTBL_JSON);
 
-    while (krb5_cc_cache_next (context, cursor, &id) == 0) {
-	krb5_principal principal = NULL;
+    while (krb5_cccol_cursor_next(context, cursor, &id) == 0) {
 	int expired = 0;
 	char *name;
 	time_t t;
 
-	ret = krb5_cc_get_principal(context, id, &principal);
-	if (ret)
-	    continue;
-
-	expired = check_for_tgt (context, id, principal, &t);
+	expired = check_expiration(context, id, &t);
 
 	ret = krb5_cc_get_friendly_name(context, id, &name);
 	if (ret == 0) {
 	    const char *str;
 	    char *fname;
+
 	    rtbl_add_column_entry(ct, COL_NAME, name);
-	    rtbl_add_column_entry(ct, COL_CACHENAME,
-				  krb5_cc_get_name(context, id));
+	    free(name);
+
 	    if (expired)
 		str = N_(">>> Expired <<<", "");
 	    else
 		str = printable_time(t);
 	    rtbl_add_column_entry(ct, COL_EXPIRES, str);
-	    free(name);
 
 	    ret = krb5_cc_get_full_name(context, id, &fname);
 	    if (ret)
 		krb5_err (context, 1, ret, "krb5_cc_get_full_name");
 
-	    if (strcmp(fname, def_name) == 0)
+	    rtbl_add_column_entry(ct, COL_CACHENAME, fname);
+	    if (opt->json_flag)
+		;
+	    else if (strcmp(fname, def_name) == 0)
 		rtbl_add_column_entry(ct, COL_DEFCACHE, "*");
 	    else
 		rtbl_add_column_entry(ct, COL_DEFCACHE, "");
@@ -539,16 +558,17 @@ list_caches(krb5_context context)
 	    krb5_xfree(fname);
 	}
 	krb5_cc_close(context, id);
-
-	krb5_free_principal(context, principal);
     }
 
-    krb5_cc_cache_end_seq_get(context, cursor);
+    krb5_cccol_cursor_free(context, &cursor);
 
     free(def_name);
     rtbl_format(ct, stdout);
     rtbl_destroy(ct);
 
+    if (opt->json_flag)
+	printf("\n");
+
     return 0;
 }
 
@@ -556,147 +576,83 @@ list_caches(krb5_context context)
  *
  */
 
-static int version_flag		= 0;
-static int help_flag		= 0;
-static int do_verbose		= 0;
-static int do_list_caches	= 0;
-static int do_all_content	= 0;
-static int do_test		= 0;
-#ifndef NO_AFS
-static int do_tokens		= 0;
-#endif
-static int do_v5		= 1;
-static char *cred_cache;
-static int do_flags	 	= 0;
-static int do_hidden	 	= 0;
-
-static struct getargs args[] = {
-    { NULL, 'f', arg_flag, &do_flags },
-    { "cache",			'c', arg_string, &cred_cache,
-      NP_("credentials cache to list", ""), "cache" },
-    { "test",			't', arg_flag, &do_test,
-      NP_("test for having tickets", ""), NULL },
-    { NULL,			's', arg_flag, &do_test },
-#ifndef NO_AFS
-    { "tokens",			'T',   arg_flag, &do_tokens,
-      NP_("display AFS tokens", ""), NULL },
-#endif
-    { "v5",			'5',	arg_flag, &do_v5,
-      NP_("display v5 cred cache", ""), NULL},
-    { "all-content",		'A', arg_flag, &do_all_content,
-      NP_("all caches with their content", ""), NULL },
-    { "list-caches",		'l', arg_flag, &do_list_caches,
-      NP_("list all caches", ""), NULL },
-    { "verbose",		'v', arg_flag, &do_verbose,
-      NP_("verbose output", ""), NULL },
-    { "hidden",			0,   arg_flag, &do_hidden,
-      NP_("display hidden credentials", ""), NULL },
-    { NULL,			'a', arg_flag, &do_verbose },
-    { NULL,			'n', arg_flag, &do_verbose },
-    { "version", 		0,   arg_flag, &version_flag,
-      NP_("print version", ""), NULL },
-    { "help",			0,   arg_flag, &help_flag,
-      NULL, NULL}
-};
-
-static void
-usage (int ret)
-{
-    arg_printusage_i18n (args,
-			 sizeof(args)/sizeof(*args),
-			 N_("Usage: ", ""),
-			 NULL,
-			 "",
-			 getarg_i18n);
-    exit (ret);
-}
-
 int
-main (int argc, char **argv)
+klist(struct klist_options *opt, int argc, char **argv)
 {
-    krb5_context context;
     krb5_error_code ret;
-    int optidx = 0;
     int exit_status = 0;
 
-    setprogname (argv[0]);
+    int do_verbose =
+	opt->verbose_flag ||
+	opt->a_flag ||
+	opt->n_flag;
+    int do_test =
+	opt->test_flag ||
+	opt->s_flag;
 
-    setlocale (LC_ALL, "");
-    bindtextdomain ("heimdal_kuser", HEIMDAL_LOCALEDIR);
-    textdomain("heimdal_kuser");
-
-    if(getarg(args, sizeof(args) / sizeof(args[0]), argc, argv, &optidx))
-	usage(1);
-
-    if (help_flag)
-	usage (0);
-
-    if(version_flag){
+    if(opt->version_flag) {
 	print_version(NULL);
 	exit(0);
     }
 
-    argc -= optidx;
-
-    if (argc != 0)
-	usage (1);
-
-    ret = krb5_init_context (&context);
-    if (ret)
-	errx (1, "krb5_init_context failed: %d", ret);
-
-
-    if (do_list_caches) {
-	exit_status = list_caches(context);
+    if (opt->list_all_flag) {
+	exit_status = list_caches(heimtools_context, opt);
 	return exit_status;
     }
 
-    if (do_v5) {
+    if (opt->v5_flag) {
 	krb5_ccache id;
 
-	if (do_all_content) {
+	if (opt->all_content_flag) {
 	    krb5_cc_cache_cursor cursor;
+	    int first = 1;
 
-	    ret = krb5_cc_cache_get_first (context, NULL, &cursor);
+	    ret = krb5_cc_cache_get_first(heimtools_context, NULL, &cursor);
 	    if (ret)
-		krb5_err (context, 1, ret, "krb5_cc_cache_get_first");
+		krb5_err(heimtools_context, 1, ret, "krb5_cc_cache_get_first");
 
+	    if (opt->json_flag)
+		printf("[");
+	    while (krb5_cc_cache_next(heimtools_context, cursor, &id) == 0) {
+		if (opt->json_flag && !first)
+		    printf(",");
 
-	    while (krb5_cc_cache_next (context, cursor, &id) == 0) {
-		exit_status |= display_v5_ccache(context, id, do_test,
-						 do_verbose, do_flags,
-						 do_hidden);
-		printf("\n\n");
+		exit_status |= display_v5_ccache(heimtools_context, id, do_test,
+						 do_verbose, opt->flags_flag,
+						 opt->hidden_flag, opt->json_flag);
+		if (!opt->json_flag)
+		    printf("\n\n");
+
+		first = 0;
 	    }
-	    krb5_cc_cache_end_seq_get(context, cursor);
-
+	    krb5_cc_cache_end_seq_get(heimtools_context, cursor);
+	    if (opt->json_flag)
+		printf("]");
 	} else {
-	    if(cred_cache) {
-		ret = krb5_cc_resolve(context, cred_cache, &id);
+	    if(opt->cache_string) {
+		ret = krb5_cc_resolve(heimtools_context, opt->cache_string, &id);
 		if (ret)
-		    krb5_err (context, 1, ret, "%s", cred_cache);
+		    krb5_err(heimtools_context, 1, ret, "%s", opt->cache_string);
 	    } else {
-		ret = krb5_cc_default (context, &id);
+		ret = krb5_cc_default(heimtools_context, &id);
 		if (ret)
-		    krb5_err (context, 1, ret, "krb5_cc_resolve");
+		    krb5_err(heimtools_context, 1, ret, "krb5_cc_resolve");
 	    }
-	    exit_status = display_v5_ccache(context, id, do_test,
-					    do_verbose, do_flags,
-					    do_hidden);
+	    exit_status = display_v5_ccache(heimtools_context, id, do_test,
+					    do_verbose, opt->flags_flag,
+					    opt->hidden_flag, opt->json_flag);
 	}
     }
 
     if (!do_test) {
 #ifndef NO_AFS
-	if (do_tokens && k_hasafs ()) {
-	    if (do_v5)
-		printf ("\n");
-	    display_tokens (do_verbose);
+	if (opt->tokens_flag && k_hasafs()) {
+	    if (opt->v5_flag)
+		printf("\n");
+	    display_tokens(opt->verbose_flag);
 	}
 #endif
     }
 
-    krb5_free_context(context);
-
     return exit_status;
 }
