@@ -27,10 +27,10 @@
  */
 
 #include "mech_locl.h"
-RCSID("$Id: gss_duplicate_name.c 21480 2007-07-10 16:32:32Z lha $");
 
-OM_uint32 gss_duplicate_name(OM_uint32 *minor_status,
-    const gss_name_t src_name,
+GSSAPI_LIB_FUNCTION OM_uint32 GSSAPI_LIB_CALL
+gss_duplicate_name(OM_uint32 *minor_status,
+    gss_const_name_t src_name,
     gss_name_t *dest_name)
 {
 	OM_uint32		major_status;
@@ -52,10 +52,10 @@ OM_uint32 gss_duplicate_name(OM_uint32 *minor_status,
 		if (major_status != GSS_S_COMPLETE)
 			return (major_status);
 		new_name = (struct _gss_name *) *dest_name;
-		
-		SLIST_FOREACH(mn, &name->gn_mn, gmn_link) {
+
+		HEIM_SLIST_FOREACH(mn, &name->gn_mn, gmn_link) {
 		    struct _gss_mechanism_name *mn2;
-		    _gss_find_mn(minor_status, new_name, 
+		    _gss_find_mn(minor_status, new_name,
 				 mn->gmn_mech_oid, &mn2);
 		}
 	} else {
@@ -65,12 +65,12 @@ OM_uint32 gss_duplicate_name(OM_uint32 *minor_status,
 			return (GSS_S_FAILURE);
 		}
 		memset(new_name, 0, sizeof(struct _gss_name));
-		SLIST_INIT(&new_name->gn_mn);
+		HEIM_SLIST_INIT(&new_name->gn_mn);
 		*dest_name = (gss_name_t) new_name;
-		
-		SLIST_FOREACH(mn, &name->gn_mn, gmn_link) {
+
+		HEIM_SLIST_FOREACH(mn, &name->gn_mn, gmn_link) {
 			struct _gss_mechanism_name *new_mn;
-			
+
 			new_mn = malloc(sizeof(*new_mn));
 			if (!new_mn) {
 				*minor_status = ENOMEM;
@@ -78,15 +78,15 @@ OM_uint32 gss_duplicate_name(OM_uint32 *minor_status,
 			}
 			new_mn->gmn_mech = mn->gmn_mech;
 			new_mn->gmn_mech_oid = mn->gmn_mech_oid;
-			
-			major_status = 
+
+			major_status =
 			    mn->gmn_mech->gm_duplicate_name(minor_status,
 				mn->gmn_name, &new_mn->gmn_name);
 			if (major_status != GSS_S_COMPLETE) {
 				free(new_mn);
 				continue;
 			}
-			SLIST_INSERT_HEAD(&new_name->gn_mn, new_mn, gmn_link);
+			HEIM_SLIST_INSERT_HEAD(&new_name->gn_mn, new_mn, gmn_link);
 		}
 
 	}
