@@ -32,10 +32,7 @@
  */
 
 #include <config.h>
-
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
+#include <roken.h>
 
 #include <engine.h>
 
@@ -88,7 +85,7 @@ ENGINE_finish(ENGINE *engine)
     if(engine->destroy)
 	(*engine->destroy)(engine);
 
-    memset(engine, 0, sizeof(engine));
+    memset(engine, 0, sizeof(*engine));
     engine->references = -1;
 
 
@@ -284,26 +281,6 @@ ENGINE_load_builtin_engines(void)
 	ENGINE_finish(engine);
 #endif
 
-#ifdef USE_HCRYPTO_IMATH
-    /*
-     * imath
-     */
-
-    engine = ENGINE_new();
-    if (engine == NULL)
-	return;
-
-    ENGINE_set_id(engine, "imath");
-    ENGINE_set_name(engine,
-		    "Heimdal crypto imath engine version " PACKAGE_VERSION);
-    ENGINE_set_RSA(engine, RSA_imath_method());
-    ENGINE_set_DH(engine, DH_imath_method());
-
-    ret = add_engine(engine);
-    if (ret != 1)
-	ENGINE_finish(engine);
-#endif
-
 #ifdef HAVE_GMP
     /*
      * gmp
@@ -359,7 +336,7 @@ ENGINE_by_dso(const char *path, const char *id)
 	    dlclose(handle);
 	    free(engine);
 	    return NULL;
-	}	
+	}
     }
 
     {
@@ -377,7 +354,7 @@ ENGINE_by_dso(const char *path, const char *id)
 	    dlclose(handle);
 	    free(engine);
 	    return NULL;
-	}	
+	}
     }
 
     ENGINE_up_ref(engine);
