@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2006 - 2008 Kungliga Tekniska Högskolan
+ * Copyright (c) 2006 - 2016 Kungliga Tekniska Högskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  *
@@ -34,25 +34,34 @@
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
+#include <roken.h>
 
 #define HC_DEPRECATED
 #define HC_DEPRECATED_CRYPTO
 
-#include <sys/types.h>
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
 #include <assert.h>
 
 #include <evp.h>
 #include <evp-hcrypto.h>
 #include <evp-cc.h>
+#if defined(_WIN32)
+#include <evp-w32.h>
+#endif
+#include <evp-pkcs11.h>
+#include <evp-openssl.h>
 
 #include <krb5-types.h>
-#include <roken.h>
 
 #ifndef HCRYPTO_DEF_PROVIDER
-#define HCRYPTO_DEF_PROVIDER hcrypto
+# ifdef __APPLE__
+#  define HCRYPTO_DEF_PROVIDER cc
+# elif __sun
+#  define HCRYPTO_DEF_PROVIDER pkcs11_hcrypto
+# elif HAVE_HCRYPTO_W_OPENSSL
+#  define HCRYPTO_DEF_PROVIDER ossl
+# else
+#  define HCRYPTO_DEF_PROVIDER hcrypto
+# endif
 #endif
 
 #define HC_CONCAT4(x,y,z,aa)	x ## y ## z ## aa
@@ -175,10 +184,13 @@ EVP_MD_CTX_destroy(EVP_MD_CTX *ctx)
 int
 EVP_MD_CTX_cleanup(EVP_MD_CTX *ctx) HC_DEPRECATED
 {
-    if (ctx->md && ctx->md->cleanup)
-	(ctx->md->cleanup)(ctx);
-    else if (ctx->md)
+    if (ctx->md && ctx->md->cleanup) {
+	int ret = (ctx->md->cleanup)(ctx->ptr);
+	if (!ret)
+	    return ret;
+    } else if (ctx->md) {
 	memset(ctx->ptr, 0, ctx->md->ctx_size);
+    }
     ctx->md = NULL;
     ctx->engine = NULL;
     free(ctx->ptr);
@@ -253,13 +265,16 @@ EVP_DigestInit_ex(EVP_MD_CTX *ctx, const EVP_MD *md, ENGINE *engine)
 	EVP_MD_CTX_cleanup(ctx);
 	ctx->md = md;
 	ctx->engine = engine;
+        if (md == NULL)
+            return 0;
 
 	ctx->ptr = calloc(1, md->ctx_size);
 	if (ctx->ptr == NULL)
 	    return 0;
     }
-    (ctx->md->init)(ctx->ptr);
-    return 1;
+    if (ctx->md == 0)
+        return 0;
+    return (ctx->md->init)(ctx->ptr);
 }
 
 /**
@@ -360,6 +375,36 @@ EVP_sha256(void)
     return EVP_DEF_OP(HCRYPTO_DEF_PROVIDER, sha256);
 }
 
+/**
+ * The message digest SHA384
+ *
+ * @return the message digest type.
+ *
+ * @ingroup hcrypto_evp
+ */
+
+const EVP_MD *
+EVP_sha384(void)
+{
+    hcrypto_validate();
+    return EVP_DEF_OP(HCRYPTO_DEF_PROVIDER, sha384);
+}
+
+/**
+ * The message digest SHA512
+ *
+ * @return the message digest type.
+ *
+ * @ingroup hcrypto_evp
+ */
+
+const EVP_MD *
+EVP_sha512(void)
+{
+    hcrypto_validate();
+    return EVP_DEF_OP(HCRYPTO_DEF_PROVIDER, sha512);
+}
+
 /**
  * The message digest SHA1
  *
@@ -385,7 +430,7 @@ EVP_sha1(void)
 
 const EVP_MD *
 EVP_sha(void) HC_DEPRECATED
-    
+
 {
     hcrypto_validate();
     return EVP_sha1();
@@ -552,10 +597,14 @@ EVP_CIPHER_CTX_init(EVP_CIPHER_CTX *c)
 int
 EVP_CIPHER_CTX_cleanup(EVP_CIPHER_CTX *c)
 {
-    if (c->cipher && c->cipher->cleanup)
-	c->cipher->cleanup(c);
+    if (c->cipher && c->cipher->cleanup) {
+	int ret = c->cipher->cleanup(c);
+	if (!ret)
+	    return ret;
+    }
     if (c->cipher_data) {
-	memset(c->cipher_data, 0, c->cipher->ctx_size);
+        if (c->cipher)
+            memset(c->cipher_data, 0, c->cipher->ctx_size);
 	free(c->cipher_data);
 	c->cipher_data = NULL;
     }
@@ -757,6 +806,10 @@ EVP_CipherInit_ex(EVP_CIPHER_CTX *ctx, const EVP_CIPHER *c, ENGINE *engine,
 	/* assume block size is a multiple of 2 */
 	ctx->block_mask = EVP_CIPHER_block_size(c) - 1;
 
+        if ((ctx->cipher->flags & EVP_CIPH_CTRL_INIT) &&
+            !EVP_CIPHER_CTX_ctrl(ctx, EVP_CTRL_INIT, 0, NULL))
+            return 0;
+
     } else if (ctx->cipher == NULL) {
 	/* reuse of cipher, but not any cipher ever set! */
 	return 0;
@@ -784,7 +837,7 @@ EVP_CipherInit_ex(EVP_CIPHER_CTX *ctx, const EVP_CIPHER *c, ENGINE *engine,
     }
 
     if (key || (ctx->cipher->flags & EVP_CIPH_ALWAYS_CALL_INIT))
-	ctx->cipher->init(ctx, key, iv, encp);
+	return ctx->cipher->init(ctx, key, iv, encp);
 
     return 1;
 }
@@ -845,7 +898,7 @@ EVP_CipherUpdate(EVP_CIPHER_CTX *ctx, void *out, int *outlen,
 	    ctx->buf_len += inlen;
 	    return 1;
 	}
-	
+
 	/* fill in local buffer and encrypt */
 	memcpy(ctx->buf + ctx->buf_len, in, left);
 	ret = (*ctx->cipher->do_cipher)(ctx, out, ctx->buf, blocksize);
@@ -863,7 +916,7 @@ EVP_CipherUpdate(EVP_CIPHER_CTX *ctx, void *out, int *outlen,
     if (inlen) {
 	ctx->buf_len = (inlen & ctx->block_mask);
 	inlen &= ~ctx->block_mask;
-	
+
 	ret = (*ctx->cipher->do_cipher)(ctx, out, in, inlen);
 	if (ret != 1)
 	    return ret;
@@ -1086,7 +1139,7 @@ EVP_des_cbc(void)
 }
 
 /**
- * The tripple DES cipher type
+ * The triple DES cipher type
  *
  * @return the DES-EDE3-CBC EVP_CIPHER pointer.
  *
@@ -1314,7 +1367,8 @@ EVP_BytesToKey(const EVP_CIPHER *type,
 	       void *keydata,
 	       void *ivdata)
 {
-    int ivlen, keylen, first = 0;
+    unsigned int ivlen, keylen;
+    int first = 0;
     unsigned int mds = 0, i;
     unsigned char *key = keydata;
     unsigned char *iv = ivdata;
