@@ -55,16 +55,19 @@ static krb5_boolean
 fail_verify_is_ok (krb5_context context,
 		   krb5_verify_init_creds_opt *options)
 {
-    if ((options->flags & KRB5_VERIFY_INIT_CREDS_OPT_AP_REQ_NOFAIL
-	 && options->ap_req_nofail != 0)
-	|| krb5_config_get_bool (context,
-				 NULL,
-				 "libdefaults",
-				 "verify_ap_req_nofail",
-				 NULL))
+
+    if (options && (options->flags & KRB5_VERIFY_INIT_CREDS_OPT_AP_REQ_NOFAIL)
+	&& options->ap_req_nofail != 0)
 	return FALSE;
-    else
-	return TRUE;
+
+    if (krb5_config_get_bool(context,
+			     NULL,
+			     "libdefaults",
+			     "verify_ap_req_nofail",
+			     NULL))
+	return FALSE;
+
+    return TRUE;
 }
 
 KRB5_LIB_FUNCTION krb5_error_code KRB5_LIB_CALL
@@ -199,11 +202,11 @@ cleanup:
 
 /**
  * Validate the newly fetch credential, see also krb5_verify_init_creds().
- * 
+ *
  * @param context a Kerberos 5 context
  * @param creds the credentials to verify
  * @param client the client name to match up
- * @param ccache the credential cache to use 
+ * @param ccache the credential cache to use
  * @param service a service name to use, used with
  *        krb5_sname_to_principal() to build a hostname to use to
  *        verify.
@@ -229,7 +232,7 @@ krb5_get_validated_creds(krb5_context context,
 	return KRB5_PRINC_NOMATCH;
     }
 
-    ret = krb5_sname_to_principal (context, NULL, service, 
+    ret = krb5_sname_to_principal (context, NULL, service,
 				   KRB5_NT_SRV_HST, &server);
     if(ret)
 	return ret;
